@@ -1,5 +1,5 @@
 ﻿/*
-Copyright (c) 2003-2011, CKSource - Frederico Knabben. All rights reserved.
+Copyright (c) 2003-2012, CKSource - Frederico Knabben. All rights reserved.
 For licensing, see LICENSE.html or http://ckeditor.com/license
 */
 
@@ -14,7 +14,22 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 
 	var whitespaces = CKEDITOR.dom.walker.whitespaces(),
 		bookmarks = CKEDITOR.dom.walker.bookmark(),
-		nonEmpty = function( node ){ return !( whitespaces( node ) || bookmarks( node ) ); };
+		nonEmpty = function( node ){ return !( whitespaces( node ) || bookmarks( node ) );},
+		blockBogus = CKEDITOR.dom.walker.bogus();
+
+	function cleanUpDirection( element )
+	{
+		var dir, parent, parentDir;
+		if ( ( dir = element.getDirection() ) )
+		{
+			parent = element.getParent();
+			while ( parent && !( parentDir = parent.getDirection() ) )
+				parent = parent.getParent();
+
+			if ( dir == parentDir )
+				element.removeAttribute( 'dir' );
+		}
+	}
 
 	CKEDITOR.plugins.list = {
 		/*
@@ -81,17 +96,22 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 				baseIndex = 0;
 			if ( !listArray || listArray.length < baseIndex + 1 )
 				return null;
-			var doc = listArray[ baseIndex ].parent.getDocument(),
+			var i,
+				doc = listArray[ baseIndex ].parent.getDocument(),
 				retval = new CKEDITOR.dom.documentFragment( doc ),
 				rootNode = null,
 				currentIndex = baseIndex,
 				indentLevel = Math.max( listArray[ baseIndex ].indent, 0 ),
 				currentListItem = null,
-				itemDir,
+				orgDir,
+				block,
 				paragraphName = ( paragraphMode == CKEDITOR.ENTER_P ? 'p' : 'div' );
 			while ( 1 )
 			{
 				var item = listArray[ currentIndex ];
+
+				orgDir = item.element.getDirection( 1 );
+
 				if ( item.indent == indentLevel )
 				{
 					if ( !rootNode || listArray[ currentIndex ].parent.getName() != rootNode.getName() )
@@ -101,15 +121,18 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 						retval.append( rootNode );
 					}
 					currentListItem = rootNode.append( item.element.clone( 0, 1 ) );
-					for ( var i = 0 ; i < item.contents.length ; i++ )
+
+					if ( orgDir != rootNode.getDirection( 1 ) )
+						currentListItem.setAttribute( 'dir', orgDir );
+
+					for ( i = 0 ; i < item.contents.length ; i++ )
 						currentListItem.append( item.contents[i].clone( 1, 1 ) );
 					currentIndex++;
 				}
 				else if ( item.indent == Math.max( indentLevel, 0 ) + 1 )
 				{
 					// Maintain original direction (#6861).
-					var orgDir = item.element.getDirection( 1 ),
-						currDir = listArray[ currentIndex - 1 ].element.getDirection( 1 ),
+					var currDir = listArray[ currentIndex - 1 ].element.getDirection( 1 ),
 						listData = CKEDITOR.plugins.list.arrayToList( listArray, null, currentIndex, paragraphMode,
 						currDir != orgDir ? orgDir: null );
 
@@ -124,35 +147,55 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 				else if ( item.indent == -1 && !baseIndex && item.grandparent )
 				{
 					if ( listNodeNames[ item.grandparent.getName() ] )
-					{
 						currentListItem = item.element.clone( false, true );
-						itemDir = item.element.getDirection( 1 );
-						item.grandparent.getDirection( 1 ) != itemDir &&
-							currentListItem.setAttribute( 'dir', itemDir );
-					}
 					else
+						currentListItem = new CKEDITOR.dom.documentFragment( doc );
+
+					// Migrate all children to the new container,
+					// apply the proper text direction.
+					var dirLoose = item.grandparent.getDirection( 1 ) != orgDir,
+						needsBlock = currentListItem.type == CKEDITOR.NODE_DOCUMENT_FRAGMENT &&
+									 paragraphMode != CKEDITOR.ENTER_BR,
+						li = item.element,
+						className = li.getAttribute( 'class' ),
+						style = li.getAttribute( 'style' );
+
+					var child, count = item.contents.length;
+					for ( i = 0 ; i < count; i++ )
 					{
-						// Create completely new blocks here.
-						if ( dir || item.element.hasAttributes() || paragraphMode != CKEDITOR.ENTER_BR )
+						child = item.contents[ i ];
+
+						if ( child.type == CKEDITOR.NODE_ELEMENT && child.isBlockBoundary() )
 						{
-							currentListItem = doc.createElement( paragraphName );
-							item.element.copyAttributes( currentListItem, { type:1, value:1 } );
-							itemDir = item.element.getDirection() || dir;
-							itemDir &&
-								currentListItem.setAttribute( 'dir', itemDir );
+							// Apply direction on content blocks.
+							if ( dirLoose && !child.getDirection() )
+								child.setAttribute( 'dir', orgDir );
 
-							// There might be a case where there are no attributes in the element after all
-							// (i.e. when "type" or "value" are the only attributes set). In this case, if enterMode = BR,
-							// the current item should be a fragment.
-							if ( !dir && paragraphMode == CKEDITOR.ENTER_BR && !currentListItem.hasAttributes() )
-								currentListItem = new CKEDITOR.dom.documentFragment( doc );
+							// Merge into child styles.
+							style && child.setAttribute( 'style',
+										 style.replace( /([^;])$/, '$1;') +
+										 ( child.getAttribute( 'style' ) || '' ) );
+
+							className && child.addClass( className );
 						}
-						else
-							currentListItem = new CKEDITOR.dom.documentFragment( doc );
-					}
+						else if ( dirLoose || needsBlock || style || className )
+						{
+							// Establish new block to hold text direction and styles.
+							if ( !block )
+							{
+								block = doc.createElement( paragraphName );
+								dirLoose && block.setAttribute( 'dir', orgDir );
+							}
 
-					for ( i = 0 ; i < item.contents.length ; i++ )
-						currentListItem.append( item.contents[i].clone( 1, 1 ) );
+							// Copy over styles to new block;
+							style && block.setAttribute( 'style', style );
+							className && block.setAttribute( 'class', className );
+
+							block.append( child.clone( 1, 1 ) );
+						}
+
+						currentListItem.append( block || child.clone( 1, 1 ) );
+					}
 
 					if ( currentListItem.type == CKEDITOR.NODE_DOCUMENT_FRAGMENT
 						 && currentIndex != listArray.length - 1 )
@@ -172,20 +215,6 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 						}
 					}
 
-					if ( currentListItem.type == CKEDITOR.NODE_ELEMENT &&
-							currentListItem.getName() == paragraphName &&
-							currentListItem.$.firstChild )
-					{
-						currentListItem.trim();
-						var firstChild = currentListItem.getFirst();
-						if ( firstChild.type == CKEDITOR.NODE_ELEMENT && firstChild.isBlockBoundary() )
-						{
-							var tmp = new CKEDITOR.dom.documentFragment( doc );
-							currentListItem.moveChildren( tmp );
-							currentListItem = tmp;
-						}
-					}
-
 					var currentListItemName = currentListItem.$.nodeName.toLowerCase();
 					if ( !CKEDITOR.env.ie && ( currentListItemName == 'div' || currentListItemName == 'p' ) )
 						currentListItem.appendBogus();
@@ -196,18 +225,29 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 				else
 					return null;
 
+				block = null;
+
 				if ( listArray.length <= currentIndex || Math.max( listArray[ currentIndex ].indent, 0 ) < indentLevel )
 					break;
 			}
 
-			// Clear marker attributes for the new list tree made of cloned nodes, if any.
 			if ( database )
 			{
-				var currentNode = retval.getFirst();
+				var currentNode = retval.getFirst(),
+					listRoot = listArray[ 0 ].parent;
+
 				while ( currentNode )
 				{
 					if ( currentNode.type == CKEDITOR.NODE_ELEMENT )
+					{
+						// Clear marker attributes for the new list tree made of cloned nodes, if any.
 						CKEDITOR.dom.element.clearMarkers( database, currentNode );
+
+						// Clear redundant direction attribute specified on list items.
+						if ( currentNode.getName() in CKEDITOR.dtd.$listItem )
+							cleanUpDirection( currentNode );
+					}
+
 					currentNode = currentNode.getNextSourceNode();
 				}
 			}
@@ -363,14 +403,13 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 				contentBlock.appendTo( listItem );
 			else
 			{
-				// Remove DIR attribute if it was merged into list root.
+				contentBlock.copyAttributes( listItem );
+				// Remove direction attribute after it was merged into list root. (#7657)
 				if ( listDir && contentBlock.getDirection() )
 				{
-					contentBlock.removeStyle( 'direction' );
-					contentBlock.removeAttribute( 'dir' );
+					listItem.removeStyle( 'direction' );
+					listItem.removeAttribute( 'dir' );
 				}
-
-				contentBlock.copyAttributes( listItem );
 				contentBlock.moveChildren( listItem );
 				contentBlock.remove();
 			}
@@ -457,6 +496,24 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 		this.type = type;
 	}
 
+	var elementType = CKEDITOR.dom.walker.nodeType( CKEDITOR.NODE_ELEMENT );
+	// Merge list items with direction preserved. (#7448)
+	function mergeListItems( from, into, refNode, toHead )
+	{
+		var child, itemDir;
+		while ( ( child = from.getFirst( elementType ) ) )
+		{
+			if ( ( itemDir = child.getDirection( 1 ) ) !== into.getDirection( 1 ) )
+				child.setAttribute( 'dir', itemDir );
+
+			child.remove();
+
+			refNode ?
+				child[ toHead ? 'insertBefore' : 'insertAfter' ]( refNode ) :
+				into.append( child, toHead  );
+		}
+	}
+
 	listCommand.prototype = {
 		exec : function( editor )
 		{
@@ -598,21 +655,24 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 					removeList.call( this, editor, groupObj, database );
 			}
 
-			// For all new lists created, merge adjacent, same type lists.
+			// For all new lists created, merge into adjacent, same type lists.
 			for ( i = 0 ; i < listsCreated.length ; i++ )
 			{
 				listNode = listsCreated[i];
 				var mergeSibling, listCommand = this;
-				( mergeSibling = function( rtl ){
+				( mergeSibling = function( rtl )
+				{
 
 					var sibling = listNode[ rtl ?
 						'getPrevious' : 'getNext' ]( CKEDITOR.dom.walker.whitespaces( true ) );
 					if ( sibling && sibling.getName &&
 						 sibling.getName() == listCommand.type )
 					{
-						sibling.remove();
 						// Move children order by merge direction.(#3820)
-						sibling.moveChildren( listNode, rtl );
+						mergeListItems( listNode, sibling, null, !rtl );
+
+						listNode.remove();
+						listNode = sibling;
 					}
 				} )();
 				mergeSibling( 1 );
@@ -687,6 +747,96 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 	for ( i in dtd.$listItem )
 		defaultListHtmlFilterRules.elements[ i ] = getExtendNestedListFilter( true );
 
+	// Check if node is block element that recieves text.
+	function isTextBlock( node )
+	{
+		return node.type == CKEDITOR.NODE_ELEMENT &&
+			   ( node.getName() in CKEDITOR.dtd.$block ||
+				 node.getName() in CKEDITOR.dtd.$listItem ) &&
+			   CKEDITOR.dtd[ node.getName() ][ '#' ];
+	}
+
+	// Merge the visual line content at the cursor range into the block.
+	function joinNextLineToCursor( editor, cursor, nextCursor )
+	{
+		editor.fire( 'saveSnapshot' );
+
+		// Merge with previous block's content.
+		nextCursor.enlarge( CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS );
+		var frag = nextCursor.extractContents();
+
+		cursor.trim( false, true );
+
+		// Kill original bogus;
+		var currentPath = new CKEDITOR.dom.elementPath( cursor.startContainer );
+		var currentLi = currentPath.lastElement.getAscendant( 'li', 1 );
+
+		var bogus = currentPath.block.getBogus();
+		bogus && bogus.remove();
+
+		// Kill the tail br in extracted.
+		var last = frag.getLast();
+		if ( last && last.type == CKEDITOR.NODE_ELEMENT && last.is( 'br' ) )
+			last.remove();
+
+		// Insert fragment at the range position.
+		var nextNode = cursor.startContainer.getChild( cursor.startOffset );
+		if ( nextNode )
+			frag.insertBefore( nextNode );
+		else
+			cursor.startContainer.append( frag );
+
+		var nextPath = new CKEDITOR.dom.elementPath( nextCursor.startContainer );
+		var nextLi = nextCursor.startContainer.getAscendant( 'li', 1 );
+
+		// Move the sub list nested in the next list item.
+		if ( nextLi )
+		{
+			var sublist = getSubList( nextLi );
+			if ( sublist )
+			{
+				// If next line is in the sub list of the current list item.
+				if ( currentLi.contains( nextLi ) )
+				{
+					mergeListItems( sublist, nextLi.getParent(), nextLi );
+					sublist.remove();
+				}
+				// Migrate the sub list to current list item.
+				else
+					currentLi.append( sublist );
+			}
+		}
+
+
+		if ( nextCursor.checkStartOfBlock() &&
+			 nextCursor.checkEndOfBlock() )
+		{
+			var nextBlock = nextPath.block,
+				parentBlock = nextBlock.getParent();
+
+			nextBlock.remove();
+
+			// Remove if the path block container is now empty, e.g. li.
+			if ( parentBlock &&
+				 !parentBlock.getFirst( nonEmpty ) &&
+				 !parentBlock.equals( nextPath.blockLimit ) )
+			{
+				parentBlock.remove();
+			}
+		}
+
+		// Make fresh selection.
+		cursor.select();
+
+		editor.fire( 'saveSnapshot' );
+	}
+
+	function getSubList( li )
+	{
+		var last = li.getLast( nonEmpty );
+		return last && last.type == CKEDITOR.NODE_ELEMENT && last.getName() in listNodeNames ? last : null;
+	}
+
 	CKEDITOR.plugins.add( 'list',
 	{
 		init : function( editor )
@@ -710,6 +860,96 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
 			// Register the state changing handlers.
 			editor.on( 'selectionChange', CKEDITOR.tools.bind( onSelectionChange, numberedListCommand ) );
 			editor.on( 'selectionChange', CKEDITOR.tools.bind( onSelectionChange, bulletedListCommand ) );
+
+			// [IE8] Fix "backspace" after list and "del" at the end of list item. (#8248)
+			if ( CKEDITOR.env.ie8Compat )
+			{
+				editor.on( 'key', function( evt )
+				{
+					var key = evt.data.keyCode;
+
+					// DEl/BACKSPACE
+					if ( editor.mode == 'wysiwyg' && key in { 8 : 1, 46 : 1 } )
+					{
+						var sel = editor.getSelection(),
+						range = sel.getRanges()[ 0 ];
+
+						if ( !range.collapsed )
+							return;
+
+						var isBackspace = key == 8;
+						var body = editor.document.getBody();
+						var walker = new CKEDITOR.dom.walker( range.clone() );
+						walker.evaluator = function( node ) { return nonEmpty( node ) && !blockBogus( node ); };
+
+						var cursor = range.clone();
+
+						if ( isBackspace )
+						{
+							walker.range.setStartAt( body, CKEDITOR.POSITION_AFTER_START );
+							walker.range.setEnd( range.startContainer, range.startOffset );
+
+							var previous = walker.previous();
+
+							// Check if cursor collapsed right behind of a list.
+							if ( previous &&
+								 previous.type == CKEDITOR.NODE_ELEMENT &&
+								 previous.getName() in listNodeNames )
+							{
+								walker.range.selectNodeContents( previous );
+								walker.reset();
+								walker.evaluator = isTextBlock;
+
+								// Place cursor at the end of previous block.
+								cursor.moveToElementEditEnd( walker.lastForward() );
+								joinNextLineToCursor( editor, cursor, range );
+								evt.cancel();
+							}
+						}
+						else
+						{
+							var li = range.startContainer.getAscendant( 'li', 1 );
+							if ( li )
+							{
+								walker.range.setEndAt( body, CKEDITOR.POSITION_BEFORE_END );
+
+								var last = li.getLast( nonEmpty );
+								var block = last && isTextBlock( last ) ? last : li;
+
+								// Indicate cursor at the visual end of an list item.
+								var isAtEnd = 0;
+
+								var next = walker.next();
+
+								// When list item contains a sub list.
+								if ( next && next.type == CKEDITOR.NODE_ELEMENT &&
+									 next.getName() in listNodeNames
+									 && next.equals( last ) )
+								{
+									isAtEnd = 1;
+
+									// Move to the first item in sub list.
+									next = walker.next();
+								}
+								// Right at the end of list item.
+								else if ( range.checkBoundaryOfElement( block, CKEDITOR.END ) )
+									isAtEnd = 1;
+
+
+								if ( isAtEnd && next )
+								{
+									// Put cursor range there.
+									var nextLine = range.clone();
+									nextLine.moveToElementEditStart( next );
+
+									joinNextLineToCursor( editor, cursor, nextLine );
+									evt.cancel();
+								}
+							}
+						}
+					}
+				} );
+			}
 		},
 
 		afterInit : function ( editor )
